import datetime


def iso_8601_duration(seconds):
    hours = seconds // 3600
    seconds = seconds % 3600
    minutes = seconds // 60
    seconds = seconds % 60

    formatted_date = "PT"
    if hours:
        formatted_date = "%s%dH" % (formatted_date, hours)
    if minutes:
        formatted_date = "%s%dM" % (formatted_date, minutes)
    if seconds or formatted_date == "PT":
        formatted_date = "%s%dS" % (formatted_date, seconds)

    return formatted_date


def timestamp(utc_dt=None):
    """
    Returns the utc timestamp of now or the given utc_dt object.
    """
    epoch = datetime.datetime.utcfromtimestamp(0)
    dt = utc_dt or datetime.datetime.utcnow()
    return (dt - epoch).total_seconds()


def ms_to_timecode(ms, fps=None):
    """
    Formats milliseconds into an hour-minute-second timecode,
    like 01:30:43 if `fps` is not set, or a hour-minute-second-frame
    timecode if fps is set, like 01:30:43:12

    When fps is set to 29.97 (NTSC), it autocorrects it to 30000/1001
    (29.97002997002997).
    """
    hours = ms // 3600000
    ms = ms - hours * 3600000
    minutes = ms // 60000
    ms = ms - minutes * 60000
    seconds = ms // 1000
    ms = ms - seconds * 1000

    if fps:
        if fps == 29.97:
            fps = 30000 / 1001.0
        frame = ms / (1000.0 / fps)
        tc = "%02d:%02d:%02d:%02d" % (hours, minutes, seconds, frame)
    else:
        tc = "%02d:%02d:%02d" % (hours, minutes, seconds)

    return tc



import datetime
import unittest

from zaach.time import formatter


class FormatterTests(unittest.TestCase):
    def test_iso_8601_duration_1(self):
        self.assertEqual(formatter.iso_8601_duration(0), "PT0S")

    def test_iso_8601_duration_2(self):
        self.assertEqual(formatter.iso_8601_duration(1), "PT1S")

    def test_iso_8601_duration_3(self):
        self.assertEqual(formatter.iso_8601_duration(60), "PT1M")

    def test_iso_8601_duration_4(self):
        self.assertEqual(formatter.iso_8601_duration(70), "PT1M10S")

    def test_iso_8601_duration_5(self):
        self.assertEqual(formatter.iso_8601_duration(3600), "PT1H")

    def test_iso_8601_duration_6(self):
        self.assertEqual(formatter.iso_8601_duration(3610), "PT1H10S")

    def test_iso_8601_duration_7(self):
        self.assertEqual(formatter.iso_8601_duration(3660), "PT1H1M")

    def test_iso_8601_duration_8(self):
        self.assertEqual(formatter.iso_8601_duration(3661), "PT1H1M1S")

    def test_timestamp_1(self):
        ts = formatter.timestamp()
        self.assertTrue(ts)

    def test_timestamp_2(self):
        ts = formatter.timestamp(datetime.datetime(1970, 1, 1))
        self.assertEqual(ts, 0)

    def test_timestamp_3(self):
        if hasattr(datetime.datetime, "timestamp"):  # python3
            ts1 = formatter.timestamp()
            ts2 = datetime.datetime.timestamp(datetime.datetime.utcnow())
            self.assertTrue((ts2 - ts1) < 1)

    def test_ms_to_timecode_1(self):
        ms = (3600 + 60 + 1) * 1000
        hms = formatter.ms_to_timecode(ms)
        self.assertEqual("01:01:01", hms)

    def test_ms_to_timecode_2(self):
        ms = (3600 * 100 + 60 * 59 + 59) * 1000
        hms = formatter.ms_to_timecode(ms)
        self.assertEqual("100:59:59", hms)

    def test_ms_to_timecode_3(self):
        ms = (3600 + 60 + 1) * 1000 + 500
        hmsf = formatter.ms_to_timecode(ms, fps=25)
        self.assertEqual("01:01:01:12", hmsf)

    def test_ms_to_timecode_4(self):
        ms = 999
        hmsf = formatter.ms_to_timecode(ms, fps=25)
        self.assertEqual("00:00:00:24", hmsf)

    def test_ms_to_timecode_5(self):
        ms = 999
        hmsf = formatter.ms_to_timecode(ms, fps=29.97)
        self.assertEqual("00:00:00:29", hmsf)


if __name__ == "__main__":
    unittest.main()



import base64


def encode(unencoded):
    """
    Encodes the argument to be safe to use in an URL and a filename.

    See RFC 4648, Section 5 "Base 64 Encoding with URL and Filename
    Safe Alphabet" for details.
    """
    return (
        base64.b64encode(unencoded.encode("utf-8"))
        .decode("utf-8")
        .rstrip("=")
        .replace("+", "-")
        .replace("/", "_")
    )


def decode(encoded):
    """
    Decodes the argument.

    See RFC 4648, Section 5 "Base 64 Encoding with URL and Filename
    Safe Alphabet" for details.
    """
    return base64.b64decode(
        "{}{}".format(
            encoded.replace("-", "+").replace("_", "/"),
            ((-1 * len(encoded)) % 4) * "=",
        )
    ).decode("utf-8")



import unittest

from zaach import strings


class StringsTestCase(unittest.TestCase):
    def test_lcut_1(self):
        s = "'foo'"
        sub = "'"
        self.assertEqual(strings.lcut(s, sub), "foo'")

    def test_lcut_2(self):
        s = ""
        sub = "'"
        self.assertEqual(strings.lcut(s, sub), "")

    def test_rcut_1(self):
        s = "'foo'"
        sub = "'"
        self.assertEqual(strings.rcut(s, sub), "'foo")

    def test_rcut_2(self):
        s = ""
        sub = "'"
        self.assertEqual(strings.rcut(s, sub), "")

    def test_cut(self):
        s = "'foo'"
        sub = "'"
        self.assertEqual(strings.cut(s, sub), "foo")



from distutils.core import setup
from setuptools import find_packages


setup(
    name="zaach",
    version="1.0.9",
    packages=find_packages(),
    description="A collection of helpers",
    author="Fabian Topfstedt",
    url="https://bitbucket.org/fabian/zaach",
    license="The MIT Licence",
    package_data={"": ["LICENCE", "README.md"]},
    long_description="A collection of helpers",
    keywords=["base64url", "timezone", "conversion", "math"],
    classifiers=[],
)



import unittest

from zaach import base64url


class Base64UrlTests(unittest.TestCase):
    def test_encode_drops_padding(self):
        unencoded = "a"
        encoded = base64url.encode(unencoded)
        expected = "YQ"
        self.assertEqual(encoded, expected)

    def test_encode_underscore(self):
        unencoded = "subjects?_d=1"
        encoded = base64url.encode(unencoded)
        expected = "c3ViamVjdHM_X2Q9MQ"
        self.assertEqual(encoded, expected)

    def test_encode_minus(self):
        unencoded = "".join([chr(i) for i in range(128)])
        encoded = base64url.encode(unencoded)
        expected = (
            "AAECAwQFBgcICQoLDA0ODxAREhMUFRYXGBkaGxwdHh8gISIjJCUmJygpKissLS4v"
            "MDEyMzQ1Njc4OTo7PD0-P0BBQkNERUZHSElKS0xNTk9QUVJTVFVWV1hZWltcXV5f"
            "YGFiY2RlZmdoaWprbG1ub3BxcnN0dXZ3eHl6e3x9fn8"
        )
        self.assertEqual(encoded, expected)

    def test_decode_adds_padding(self):
        encoded = "YQ"
        unencoded = base64url.decode(encoded)
        expected = "a"
        self.assertEqual(unencoded, expected)

    def test_decode_underscore(self):
        encoded = "c3ViamVjdHM_X2Q9MQ"
        unencoded = base64url.decode(encoded)
        expected = "subjects?_d=1"
        self.assertEqual(unencoded, expected)

    def test_decode_minus(self):
        encoded = (
            "AAECAwQFBgcICQoLDA0ODxAREhMUFRYXGBkaGxwdHh8gISIjJCUmJygpKissLS4v"
            "MDEyMzQ1Njc4OTo7PD0-P0BBQkNERUZHSElKS0xNTk9QUVJTVFVWV1hZWltcXV5f"
            "YGFiY2RlZmdoaWprbG1ub3BxcnN0dXZ3eHl6e3x9fn8"
        )
        unencoded = base64url.decode(encoded)
        expected = "".join([chr(i) for i in range(128)])
        self.assertEqual(unencoded, expected)
